unit dsddraw;
{*************TDSDDraw************
Extremely specialized Direct3D/DirectDraw wrapper
Made in Delphi 4

Author:  Dag Sverre Seljebotn, dagsverre@sensewave.com
Created: 25-04-2000
History: 25-04-2000 First release

Needs DirectX 7.0 headers for Delphi. You can find them on www.delphi-jedi.org,
under the Graphics section (bottom-left...)

Created for the FF8 Configurator v. 2, http://ffsf.cjb.net

This code is distributed under the GNU General Public License.
NO WARRANTY. See COPYING for details.

Copyright (c) 2000 Dag Sverre Seljebotn
}

interface
uses ActiveX, Classes, DirectDraw, Direct3D;

type
  TDDrawCaps = record
    DeviceID: TGUID;
    Hardware: Boolean;
  end;
  PDDrawCaps = ^TDDrawCaps;

  TDSDDraw = class
  private
    FDeviceList: TStringList;
    function GetDeviceList: TStringList;
    function GetDeviceID(I: Integer): TDDrawCaps;
  public
    destructor Destroy; override;

    function IndexOfDevice(ID: TGUID): Integer;

    property DeviceList: TStringList read GetDeviceList;
    property DeviceCaps[I: Integer]: TDDrawCaps read GetDeviceID;
  end;

implementation

{ TDSDDraw }

destructor TDSDDraw.Destroy;
var
  I: Integer;
begin
  for I:=0 to FDeviceList.Count - 1 do
    FreeMem(Pointer(FDeviceList.Objects[I]));
  FDeviceList.Free;
  inherited Destroy;
end;

function TDSDDraw.GetDeviceID(I: Integer): TDDrawCaps;
begin
  Result:=PDDrawCaps(DeviceList.Objects[I])^;
end;

function EnumD3DCallback(lpGuid: PGUID;
      lpDeviceDescription: PAnsiChar; lpDeviceName: PAnsiChar;
      var lpD3DHWDeviceDesc: TD3DDeviceDesc;
      var lpD3DHELDeviceDesc: TD3DDeviceDesc;
      lpContext : Pointer) : HResult; stdcall;
begin
  if lpD3DHWDeviceDesc.dcmColorModel <> 0 then
  begin
    Boolean(lpContext^):=True;
    Result:=D3DENUMRET_CANCEL
  end
  else
    Result:=D3DENUMRET_OK;
end;

function EnumDDCallback(lpGUID: PGUID; lpDriverDescription: PAnsiChar;
      lpDriverName: PAnsiChar; lpContext: Pointer) : Longbool; stdcall;
const
  EmptyGUID: TGUID = '{00000000-0000-0000-0000-000000000000}';
var
  BufCaps: PDDrawCaps;
  DirectDraw: IDirectDraw;
begin
  GetMem(BufCaps,SizeOf(TDDrawCaps));
  if lpGUID <> nil then
    BufCaps^.DeviceID:=lpGUID^
  else
    BufCaps^.DeviceID:=EmptyGUID;

  DirectDrawCreate(lpGUID, DirectDraw, nil);
  (DirectDraw as IDirect3D).EnumDevices(@EnumD3DCallback, @BufCaps^.Hardware);  //Enum inside enum...funny thing


  TStrings(lpContext).AddObject(lpDriverDescription, TObject(BufCaps));
  Result:=True;
end;

function TDSDDraw.GetDeviceList: TStringList;
begin
  if FDeviceList = nil then
  begin
    FDeviceList:=TStringList.Create;
    DirectDrawEnumerate(@EnumDDCallback, FDeviceList);
  end;
  Result:=FDeviceList;
end;

function TDSDDraw.IndexOfDevice(ID: TGUID): Integer;
var
  I: Integer;
begin
  Result:=-1;
  for I:=0 to FDeviceList.Count - 1 do
    if IsEqualIID(PGUID(FDeviceList[I])^, ID) then
    begin
      Result:=I;
      Break;
    end;
end;

end.
