unit dsdinput;
{*************TDSDinput************
Extremely specialized DirectInput wrapper
Made in Delphi 4

Author:  Dag Sverre Seljebotn, dagsverre@sensewave.com
Created: 25-04-2000
History: 26-04-2000 First release

Needs DirectX 7.0 headers for Delphi. You can find them on www.delphi-jedi.org,
under the Graphics section (bottom-left...)

Created for the FF8 Configurator v. 2, http://ffsf.cjb.net

This code is distributed under the GNU General Public License.
NO WARRANTY. See COPYING for details.

Copyright (c) 2000 Dag Sverre Seljebotn
}

interface
uses DirectInput;

type
  TDeviceTypes = (dtJoysticks, dtKeyboard);
  TDevices = set of TDeviceTypes;

  TDeviceArray = array of IDirectInputDevice2;
  PDeviceArray = ^TDeviceArray;

  TDSDInput = class
  private
    FInput: IDirectInput;
    FJoysticks: TDeviceArray;
    FKeyboard: IDirectInputDevice;
    FHandle: Cardinal;
    function GetIsMonitoring: Boolean;
    function GetJoyCount: Integer;
    function GetJoyState(Device: Integer): TDIJoyState;
    function GetKeyState: TDIKeyboardState;
  protected
  public
    constructor Create(Handle: Cardinal);
    destructor Destroy; override;
    procedure StartMonitoring(Devices: TDevices);
    procedure StopMonitoring;

    property JoyState[Device: Integer]: TDIJoyState read GetJoyState;
    property KeyState: TDIKeyboardState read GetKeyState;
    property IsMonitoring: Boolean read GetIsMonitoring;
    property JoyCount: Integer read GetJoyCount;
  end;

implementation
uses ActiveX;

{ TDSDInput }

constructor TDSDInput.Create(Handle: Cardinal);
begin
  inherited Create;
  FHandle:=Handle;
end;

destructor TDSDInput.Destroy;
begin
  StopMonitoring;
  inherited Destroy;
end;

procedure TDSDInput.StopMonitoring;
var
  I: Integer;
begin
  FInput:=nil;
  FKeyboard:=nil;
  for I:=Low(FJoySticks) to High(FJoySticks) do    //Is this done automatically by the compiler???
  begin
    FJoysticks[I].Unacquire;
    FJoysticks[I]:=nil;
  end;
  SetLength(FJoySticks, 0);
end;

function TDSDInput.GetIsMonitoring: Boolean;
begin
  Result:=(FInput <> nil) and (Length(FJoysticks) <> 0);
end;

function TDSDInput.GetJoyCount: Integer;
begin
  Result:=Length(FJoysticks);
end;

type
  PDSDInput = ^TDSDInput;

function JoyEnumCallback (var lpddi: TDIDeviceInstanceA; lParam: PDSDInput): Integer; stdcall;
var
  Count: Integer;
begin
  Count:=Length(lParam^.FJoysticks);
  SetLength(lParam^.FJoysticks, Count + 1);

  lParam^.FInput.CreateDevice(lpddi.guidInstance, IDirectInputDevice(lParam^.FJoysticks[Count]), nil);
  with lParam^.FJoysticks[Count] do
  begin
    SetDataFormat(c_dfDIJoystick);
    SetCooperativeLevel(lParam^.FHandle, DISCL_FOREGROUND or DISCL_NONEXCLUSIVE);
    Acquire;
  end;
  Result:=DIENUM_CONTINUE;
end;

procedure TDSDInput.StartMonitoring(Devices: TDevices);
begin
  DirectInputCreate(hInstance, DIRECTINPUT_VERSION, FInput, nil);

  if dtJoysticks in Devices then
    FInput.EnumDevices(DIDEVTYPE_JOYSTICK, @JoyEnumCallback, @Self, DIEDFL_ATTACHEDONLY);
  if dtKeyboard in Devices then
  begin
    FInput.CreateDevice(GUID_SysKeyboard, FKeyboard, nil);
    FKeyboard.SetDataFormat(c_dfDIKeyboard);
    FKeyboard.SetCooperativeLevel(FHandle, DISCL_FOREGROUND or DISCL_NONEXCLUSIVE);
    FKeyboard.Acquire;
  end;
end;


function TDSDInput.GetJoyState(Device: Integer): TDIJoyState;
var
  Res: Cardinal;
begin
  if not IsMonitoring then
  begin
    FillChar(Result, SizeOf(Result), 0);
    Exit;
  end;
  with FJoysticks[Device] do
  begin
    Poll;
    Res:=GetDeviceState(SizeOf(Result),@Result);
    if Res = DIERR_INPUTLOST then
    begin
      Acquire;
      Res:=GetDeviceState(SizeOf(Result),@Result);
    end;
    if Failed(Res) then
      FillChar(Result, SizeOf(Result), 0);
  end;
end;

function TDSDInput.GetKeyState: TDIKeyboardState;
var
  Res: Cardinal;
begin
  if not IsMonitoring then
  begin
    FillChar(Result, SizeOf(Result), 0);
    Exit;
  end;
  with FKeyboard do
  begin
    Res:=GetDeviceState(SizeOf(Result),@Result);
    if Failed(Res) then
    begin
      Acquire;
      Res:=GetDeviceState(SizeOf(Result),@Result);
      if Failed(Res) then
        FillChar(Result, SizeOf(Result), 0);
    end;
  end;
end;

end.
