unit main;


{*************TMainForm************
The mainform for FF8Configurator.exe
Made in Delphi 4

Author:  Dag Sverre Seljebotn, dagsverre@sensewave.com
Created: 20-03-2000
History: 20-03-2000 First release

Needs DirectX 7.0 headers for Delphi. You can find them on www.delphi-jedi.org,
under the Graphics section (bottom-left...)

To get smaller file-size, I used the UPX packer.

This is a further development of FF8MIDIConfigurator.
I have now included all the configurations, because the 'official' config
overwrites the setting FF8MIDIConf made in the registry.

This code is distributed under the GNU General Public License.
NO WARRANTY. See COPYING for details.

Copyright (c) 2000 Dag Sverre Seljebotn
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ComCtrls, ExtCtrls, StdCtrls, CheckLst,
  config, dsdmusic, dsdsound, dsddraw, dsdinput, packs, jpeg;

const
  CM_CONTROLSETTINGS = WM_USER + 1;

type
  TMainForm = class(TForm)
    chkSoftware: TRadioButton;
    chkHardware: TRadioButton;
    chkQuarterScreen: TCheckBox;
    chkLoresVideo: TCheckBox;
    chkLoresFonts: TCheckBox;
    chkNoMapColor: TCheckBox;
    chkNoMapFog: TCheckBox;
    chkNoMapTexture: TCheckBox;
    chkNoBilinear: TCheckBox;
    LoresFont: TImage;
    Fog: TImage;
    Worldmap: TImage;
    WorldmapDefault: TImage;
    ColorAnim1: TImage;
    ColorAnim2: TImage;
    ColorAnim3: TImage;
    ColorAnim4: TImage;
    TextureAnim1: TImage;
    TextureAnim2: TImage;
    Animation: TTimer;
    GraphicsDevices: TComboBox;
    CheckStatus: TTimer;
    Memo: TMemo;
    chkReverseSpeakers: TCheckBox;
    chkEAX: TCheckBox;
    SoundDevices: TComboBox;
    SoundFonts: TListBox;
    StopButton: TSpeedButton;
    PlayButton: TSpeedButton;
    PlayStatus: TLabel;
    chkChorus: TCheckBox;
    chkReverb: TCheckBox;
    Tracker: TTrackBar;
    Tunes: TListBox;
    MusicDevices: TListBox;
    MusicSets: TCheckListBox;
    ControlsPage: TTabSheet;
    NoBilinear: TImage;
    OpenDialog: TOpenDialog;
    RemoveButton: TSpeedButton;
    WaitPanel: TPanel;
    K1: TEdit;
    J1: TEdit;
    K0: TEdit;
    J0: TEdit;
    K3: TEdit;
    J3: TEdit;
    Pages: TPageControl;
    procedure PreviewFont(Sender: TObject);
    procedure PreviewFog(Sender: TObject);
    procedure PreviewAnim(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PreviewQuarterScreen(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Play(Sender: TObject);
    procedure Stop(Sender: TObject);
    procedure Pause(Sender: TObject);
    procedure UpdatePlayState(Sender: TObject);
    procedure TrackerMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TrackerMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TunesDblClick(Sender: TObject);
    procedure ChangeMusicDevices(Sender: TObject);
    procedure PlaySound(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DrawPanLine(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure LoadText(Sender: TObject);
    procedure GraphicsEnter(Sender: TObject);
    procedure GraphicsExit(Sender: TObject);
    procedure AddPack(Sender: TObject);
    procedure LaunchURL(Sender: TObject);
    procedure MoveSetUp(Sender: TObject);
    procedure MoveSetDown(Sender: TObject);
    procedure CheckHardware3D(Sender: TObject);
    procedure Cancel(Sender: TObject);
    procedure WriteSettings(Sender: TObject);
    procedure ControlsEnter(Sender: TObject);
    procedure ControlsExit(Sender: TObject);
    procedure RenderChange(Sender: TObject);
    procedure NullifyKey(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LoadDefaultControls(Sender: TObject);
    procedure chkNoBilinearClick(Sender: TObject);
    procedure RemovePack(Sender: TObject);
    procedure PackEnter(Sender: TObject);
    procedure PackExit(Sender: TObject);
  private
    FMapAnim: Integer;
    FOldPan: Integer;

    FDSDMusic: TDSDMusic;
    FDSDSound: TDSDSound;
    FDSDDraw : TDSDDraw;
    FDSDInput: TDSDInput;

    FSettings: TFF8ConfigIni;
    FPacks: TPackManager;

    FCurrentTune: String;

    procedure EnablePlayControls(Value: Boolean);
    procedure Error(S: String; E: Exception);
    procedure ControlSettings(Sender: TObject; var Done: Boolean);
    procedure MoveMusicSet(Direction: Integer);
    procedure ReadSettings;
    procedure Install;

    procedure CreateDesktopIcon(const Path, DestFile: String);

    procedure ExtractResource(const ResName, DestPath: String);

    procedure EnumPacks(SF, SGT: Boolean);

    function BrowseCallBack(var AFilename: String): Boolean;
  end;

var
  MainForm: TMainForm;

implementation
{$R *.DFM}
{$R DATA.RES}

uses ActiveX, ComObj, Registry, DirectMusic, DirectSound, DirectInput, MMSystem, EAX, ShellAPI, launcher, installpath, filectrl, shlobj;

const
  EmptyGUID: TGUID = '{00000000-0000-0000-0000-000000000000}';

  SGTDelimiter = '|';

  ButtonNames: array[0..255] of String = ('',
    'Escape', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=',
    'Backspace', 'Tab', 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P',
    '[', ']', 'Return', 'Left Ctrl', 'A', 'S', 'D',
    'F', 'G', 'H', 'J', 'K', 'L', ';', '''', 'Grave', 'Left Shift',
    '\', 'Z', 'X', 'C', 'V', 'B', 'N', 'M', 'Comma', 'Period', '/',
    'Right Shift', '*', 'Left Alt', 'Space', 'Caps Lock', 'F1', 'F2', 'F3',
    'F4', 'F5', 'F6', 'F7', 'F8', 'F9', 'F10', 'Num Lock', 'Scroll Lock',
    'Num 7', 'Num 8', 'Num 9', 'Num -', 'Num 4', 'Num 5', 'Num 6', 'Num +',
    'Num 1', 'Num 2', 'Num 3', 'Num 0', 'Num .', '', '', '', 'F11', 'F12',
    '', '', '', '', '', '', '', '', '', '', '', 'F13', 'F14', 'F15', '',
    '', '', '', '', '', '', '', '', 'Kana', '', '', '', '', '', '', '', '',
    'Convert', 'Noconvert', '', 'Yen', '', '', '', '', '', '', '', '', '',
    '', '', '', '', '', '', '', 'Num Equals', '', '', 'Circumflex', '@', ':',
    '_', 'Kanji', 'Stop', 'Ax', 'Unlabeled', '', '', '', '', 'Num Enter',
    'Right Ctrl', '', '', '', '', '', '', '', '', '', '', '', '', '', '',
    '', '', '', '', '', '', '', 'Num ,', '', 'Num /', '', 'Print Scrn', 'Right Alt',
    '', '', '', '', '', '', '', '', '', '', '', '', 'Pause', '', 'Home', 'Up',
    'Page Up', '', 'Left', '', 'Right', '', 'End', 'Down', 'Page Dn', 'Insert',
    'Delete', 'Left Win', 'Right Win', 'AppMenu', 'Power', 'Sleep', '', '',
    '', '', '', '', '', 'B1', 'B2', 'B3', 'B4', 'B5', 'B6', 'B7', 'B8', 'B9',
    'B10', 'B11', 'B12', 'B13', 'B14', 'B15', 'B16', 'B17', 'B18', 'B19', 'B20',
    'B21', 'B22', 'B23', 'B24', 'B25', 'B26', 'B27', 'B28', 'UP', 'DN', 'LFT', 'RGT'
    );

procedure TMainForm.PreviewFont(Sender: TObject);
begin
  LoresFont.Visible:=TCheckbox(Sender).Checked
end;

procedure TMainForm.PreviewFog(Sender: TObject);
begin
  if TCheckbox(Sender).Checked then //was fogged, so restore old image
    Worldmap.Picture:=WorldmapDefault.Picture
  else
    Worldmap.Picture.Bitmap.Canvas.Draw(0,0,Fog.Picture.Graphic);

  PreviewAnim(nil);
end;

procedure TMainForm.PreviewAnim(Sender: TObject);
var
  Color, Texture: TImage;
begin
  Inc(FMapAnim);
  if FMapAnim > 5 then
    FMapAnim:=0;
  Color:=nil;
  if chkNoMapColor.Checked then
    Color:=ColorAnim1
  else
    case FMapAnim of
    0: Color:=ColorAnim1;
    1: Color:=ColorAnim2;
    2: Color:=ColorAnim3;
    3: Color:=ColorAnim4;
    4: Color:=ColorAnim3;
    5: Color:=ColorAnim2;
    end;
  Texture:=nil;
  if chkNoMapTexture.Checked then
    Texture:=nil
  else
    case FMapAnim of
    0: Texture:=TextureAnim1;
    1: Texture:=TextureAnim2;
    2: Texture:=TextureAnim1;
    3: Texture:=TextureAnim2;
    4: Texture:=TextureAnim1;
    5: Texture:=TextureAnim2;
    end;
  if Color <> nil then
    Worldmap.Picture.Bitmap.Canvas.Draw(0,Worldmap.Picture.Bitmap.Height -
                                        Color.Picture.Bitmap.Height,
                                        Color.Picture.Graphic);
  if Texture <> nil then
    Worldmap.Picture.Bitmap.Canvas.Draw(0,Worldmap.Picture.Bitmap.Height -
                                        Texture.Picture.Bitmap.Height,
                                        Texture.Picture.Graphic);
end;


function EnumCallback(lpGUID: PGUID; lpDriverDescription: PAnsiChar;
      lpDriverName: PAnsiChar; lpContext: Pointer) : Bool; stdcall;
var
  BufGUID: PGUID;
begin
  GetMem(BufGUID,SizeOf(TGUID));
  if lpGUID <> nil then
    BufGUID^:=lpGUID^
  else
    BufGUID^:=EmptyGUID;

  TStrings(lpContext).AddObject(lpDriverDescription, TObject(BufGUID));

  Result:=True;
end;


type
  TTrackBarWithMouse = class(TTrackBar)
  published
    property OnMouseDown;
    property OnMouseUp;
  end;

procedure TMainForm.FormCreate(Sender: TObject);
  function FindGUID(List: TStrings; GUID: TGUID): Integer;
  var
    I: Integer;
  begin
    Result:=-1;
    for I:=0 to List.Count - 1 do
      if IsEqualGUID(PGUID(List.Objects[I])^, GUID) then
      begin
        Result:=I;
        Break;
      end;
  end;

  procedure EnumTunes;
  var
    SR: TSearchRec;
    Ret: Integer;
  begin
    Ret:=FindFirst(FSettings.DataPath + '*.sgt', faArchive, SR);
    while Ret = 0 do
    begin
      Tunes.Items.Add(Copy(SR.Name,1,Length(SR.Name) - 4));
      Ret:=FindNext(SR);
    end;
  end;

begin
  CoInitialize(nil);
  Install;


  FDSDDraw:=TDSDDraw.Create;
  FDSDMusic:=TDSDMusic.Create;
  FDSDSound:=TDSDSound.Create(Handle);
  FDSDInput:=TDSDInput.Create(Handle);
  FSettings:=TFF8ConfigIni.Create(ExtractFilePath(ParamStr(0)) + 'config.ini');
  if FSettings.AddonPath = '' then
    FPacks:=TPackManager.Create('dummystring') //Never created to disk
  else
    FPacks:=TPackManager.Create(FSettings.AddonPath + 'packages.ini');

  GraphicsDevices.Items.Assign(FDSDDraw.DeviceList);
  SoundDevices.Items.Assign(FDSDSound.DeviceList);
  MusicDevices.Items.Assign(FDSDMusic.DeviceList);


  EnumPacks(True, True);
  EnumTunes;

  ReadSettings;
  CheckHardware3D(Sender);

  WorldMap.Picture:=WorldmapDefault.Picture;
  PreviewFog(chkNoMapFog);
  ChangeMusicDevices(Sender);
  with TTrackBarWithMouse(Tracker) do
  begin
    OnMouseUp:=TrackerMouseUp;
    OnMouseDown:=TrackerMouseDown;
  end;
  FOldPan:= -1;

  chkEAX.Enabled:=@EAXDirectSoundCreate <> nil;
  LoadText(nil);
end;


procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FPacks.Free;
  FSettings.Free;
  FDSDMusic.Free;
  FDSDSound.Free;
  FDSDDraw.Free;
  FDSDInput.Free;
end;

procedure TMainForm.PreviewQuarterScreen(Sender: TObject);
begin
  chkLoresFonts.Enabled:=not chkQuarterScreen.Checked;
  chkLoresFonts.Checked:=chkQuarterScreen.Checked;
  if chkQuarterScreen.Checked then
  begin
    Worldmap.Width:=100;
    Worldmap.Height:=70;
    Worldmap.Top:=207;
    Worldmap.Left:=368;
  end
  else
  begin
    Worldmap.Width:=200;
    Worldmap.Height:=140;
    Worldmap.Top:=170;
    Worldmap.Left:=316;
  end;
end;

procedure TMainForm.Error(S: String; E: Exception);
begin
  MessageBox(Handle,PChar(S + #10#10'Error: "' + E.Message + '"'),
             PChar('Error - ' + Application.Title),MB_OK or MB_ICONEXCLAMATION);
end;

procedure TMainForm.EnablePlayControls(Value: Boolean);
begin
  CheckStatus.Enabled:=Value;
  StopButton.Enabled:=Value;
  Tracker.Enabled:=Value;
  MusicDevices.Enabled:=not Value;
  chkChorus.Enabled:=not Value;
  chkReverb.Enabled:=not Value;
end;

procedure TMainForm.Play(Sender: TObject);
var
  Flags: Cardinal;
  S,
  Song,
  DLS: String;
  Overrides: TStringList;
  I: Integer;
  Flag: Boolean;
begin
  PlayStatus.Caption:='Loading...';
  PlayStatus.Refresh;
  Screen.Cursor:=crHourglass;
  try
    if FDSDMusic.IsPlaying then
      FDSDMusic.ResumeTrack
    else
    begin  //Start playing
      Flags:=0;
      if chkReverb.Checked then
        Flags:=Flags or DMUS_EFFECT_REVERB;
      if chkChorus.Checked then
        Flags:=Flags or DMUS_EFFECT_CHORUS;
      FCurrentTune:=Tunes.Items[Tunes.ItemIndex];

      //Get overrides
      for I:=0 to MusicSets.Items.Count - 1 do
        if MusicSets.Checked[I] then
          S:=S + FPacks.SGTName[Integer(MusicSets.Items.Objects[I])] + SGTDelimiter;
      Overrides:=TStringList.Create;
      try
        FPacks.GetSGTOverrides(Overrides, S);
        Song:=FCurrentTune + '.sgt';
        S:=Overrides.Values[Song];
      finally
        Overrides.Free;
      end;
      if S = '' then
        Flag:=False
      else
        repeat
          I:=Pos('|', S);
          if I <= 0 then I:=Length(S) + 1;
          Song:=Copy(S, 1, I - 1);
          Delete(S, 1, I);
          Song:=FSettings.AddonPath + Song;
          Flag:=FileExists(Song);
        until Flag or (S = '');

      if not Flag then
        Song:=FSettings.DataPath + FCurrentTune + '.sgt';

      if Soundfonts.ItemIndex < 0 then
        Abort;
      DLS:=FPacks.SFFile[Integer(Soundfonts.Items.Objects[Soundfonts.ItemIndex])];
      if DLS = '' then
        DLS:=FSettings.DataPath + 'ff8.dls';

      FDSDMusic.PlayTrack(MusicDevices.ItemIndex, Song, DLS, Flags);
    end;

    PlayStatus.Caption:='Playing ' + FCurrentTune;
    CheckStatus.Enabled:=True;

    Tracker.Max:=FDSDMusic.TrackLength;

    EnablePlayControls(True);
    PlayButton.SendToBack;
    Screen.Cursor:=crDefault;
  except
    Screen.Cursor:=crDefault;
    PlayStatus.Caption:='Failed!';
  end;
end;

procedure TMainForm.Stop(Sender: TObject);
begin
  FDSDMusic.StopTrack;
  CheckStatus.Enabled:=False;
  Tracker.Position:=0;
  PlayButton.BringToFront;
  PlayStatus.Caption:='';
  EnablePlayControls(False);
end;

procedure TMainForm.Pause(Sender: TObject);
begin
  FDSDMusic.PauseTrack;
  PlayButton.BringToFront;
end;

procedure TMainForm.UpdatePlayState(Sender: TObject);
begin
  if not FDSDMusic.IsPaused then
    Tracker.Position:=FDSDMusic.TrackPosition;
end;

procedure TMainForm.TrackerMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDSDMusic.PauseTrack;
end;

procedure TMainForm.TrackerMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDSDMusic.TrackPosition:=Tracker.Position;
  FDSDMusic.ResumeTrack;
end;

procedure TMainForm.TunesDblClick(Sender: TObject);
begin
  Stop(StopButton);
  Play(PlayButton);
end;

procedure TMainForm.ChangeMusicDevices(Sender: TObject);
begin
  chkReverb.Enabled:=(FDSDMusic.DeviceCaps[MusicDevices.ItemIndex].dwEffectFlags
      and DMUS_EFFECT_REVERB) = DMUS_EFFECT_REVERB;

  chkChorus.Enabled:=(FDSDMusic.DeviceCaps[MusicDevices.ItemIndex].dwEffectFlags
    and DMUS_EFFECT_CHORUS) = DMUS_EFFECT_CHORUS;

  chkReverb.Checked:=chkReverb.Enabled;
  chkChorus.Checked:=chkChorus.Enabled;

  
end;

procedure TMainForm.PlaySound(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Wave: TResourceStream;
  SndFormat: TWaveFormatEx;
  Pan: Integer;
begin
  Screen.Cursor:=crHourGlass;
  try
    try
      ///////////////Format of wave file in resource///////////

      FillChar(SndFormat, SizeOf(SndFormat), 0);
      with SndFormat do
      begin
        wFormatTag:=WAVE_FORMAT_PCM;
        nChannels:=1;
        nSamplesPerSec:=44100;
        wBitsPerSample:=16;
        nBlockAlign:=wBitsPerSample div 8 * nChannels;
        nAvgBytesPerSec:=nSamplesPerSec * nBlockAlign;
      end;

      if X < 20 then
        Pan:=DSBPAN_LEFT
      else if X > TControl(Sender).Width - 20 then
        Pan:=DSBPAN_RIGHT
      else
      begin
        Pan:=(TControl(Sender).Width - 60) div 2;
        Pan:=(X - Pan) * (1000 div Pan);
      end;
      if chkReverseSpeakers.Checked then
        Pan:= - Pan;

      Wave:=TResourceStream.Create(hInstance,'SoundTest',RT_RCDATA);
      try
        FDSDSound.PlaySound(SoundDevices.ItemIndex, Wave, SndFormat,
          Pan, chkEAX.Checked);
      finally
        Wave.Free;
      end;
    except
      on E: Exception do Error('Testing of device failed!', E);
    end;
  finally
    Screen.Cursor:=crDefault;
  end;
end;

procedure TMainForm.DrawPanLine(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  with (Sender as TImage).Picture.Bitmap.Canvas do
  begin
    if FOldPan < 0 then
    begin
      Pen.Mode:=pmNot;
      Pen.Width:=3;
    end
    else
    begin
      MoveTo(FOldPan, 0);
      LineTo(FOldPan, TControl(Sender).Height);
    end;
    MoveTo(X, 0);
    LineTo(X, TControl(Sender).Height);
    FOldPan:=X;
  end;
end;

procedure TMainForm.LoadText(Sender: TObject);
var
  Name: String;
  ResStream: TResourceStream;
begin
  if (Sender = nil) or (TControl(Sender).Name <> 'LicenseButton') then
    Name:='Readme'
  else //Default
    Name:='Copying';
  ResStream:=TResourceStream.Create(hInstance, Name, RT_RCDATA);
  try
    Memo.Lines.LoadFromStream(ResStream);
  finally
    ResStream.Free;
  end;
end;

procedure TMainForm.ControlSettings(Sender: TObject; var Done: Boolean);
  function DecodeJoyData(JoyState: TDIJoyState): Integer;
  const
    Threshold = 16000;
    joyUp     = 252;
    joyDown   = 253;
    joyLeft   = 254;
    joyRight  = 255;
  var
    I: Integer;
  begin
    Result:=0;
    for I:=Low(JoyState.rgbButtons) to High(JoyState.rgbButtons) do
      if (JoyState.rgbButtons[I] and $80) = $80 then
      begin
        Result:=I + 224;
        Exit;
      end;

    if (JoyState.lX - High(Smallint)) > Threshold then
      Result:= joyRight
    else if (High(Smallint) - JoyState.lX) > Threshold then
      Result:= joyLeft
    else if (JoyState.lY - High(Smallint)) > Threshold then
      Result:= joyDown
    else if (High(Smallint) - JoyState.lY) > Threshold then
      Result:= joyUp;
  end;

  function DecodeKeyData(KeyState: TDIKeyboardState): Integer;
  var
    I: Integer;
  begin
    Result:=0;
    for I:=Low(KeyState) to High(KeyState) do
      if (KeyState[I] and $80) = $80 then
      begin
        Result:=I;
        Break;
      end;
  end;

var
  Edit: TEdit;

  procedure AssignButton(Button: Integer);
  var
    I: Integer;
  begin
    if Button > 0 then
      with Edit do
      begin
        for I:=0 to ControlsPage.ControlCount - 1 do
          if ControlsPage.Controls[I].Tag = Button then
          begin
            ControlsPage.Controls[I].Tag:=0;
            (ControlsPage.Controls[I] as TEdit).Text:='';
            Break;
          end;
        Tag:=Button;
        Text:=ButtonNames[Button];
      end
  end;
var
  I: Integer;
begin
  if ActiveControl is TEdit then
    Edit:=TEdit(ActiveControl)
  else
    Exit; //Done = true...
  if Edit.Name[1] = 'J' then
      for I:=0 to FDSDInput.JoyCount - 1 do
        AssignButton(DecodeJoyData(FDSDInput.JoyState[I]))
  else if Edit.Name[1] = 'K' then
      AssignButton(DecodeKeyData(FDSDInput.KeyState))
  else
    Exit;
  Done:=False;
end;

procedure TMainForm.ControlsEnter(Sender: TObject);
begin
  Screen.Cursor:=crHourglass;
  WaitPanel.Caption:='Connecting to joysticks. Please wait...';
  WaitPanel.Visible:=True;
  Application.ProcessMessages;
  try
    FDSDInput.StartMonitoring([dtJoysticks, dtKeyboard]);
  finally
    Screen.Cursor:=crDefault;
    WaitPanel.Visible:=False;
  end;
  OnKeyDown:=NullifyKey;
  Application.OnIdle:=ControlSettings;
end;

procedure TMainForm.ControlsExit(Sender: TObject);
begin
  OnKeyDown:=nil;
  Application.OnIdle:=nil;
  FDSDInput.StopMonitoring;
end;

procedure TMainForm.GraphicsEnter(Sender: TObject);
begin
  Animation.Enabled:=True;
end;

procedure TMainForm.GraphicsExit(Sender: TObject);
begin
  Animation.Enabled:=False;
end;

procedure TMainForm.AddPack(Sender: TObject);
  procedure InstallPack;
  var
    Dll: TResourceStream;
    DllPath: String;
  begin
    DllPath:=ExtractFilePath(ParamStr(0)) + 'unrardll.~01';
    try
      Dll:=TResourceStream.Create(hInstance, 'UnRar', RT_RCDATA);
      try
        with TFileStream.Create(DllPath, fmCreate) do
        try
          CopyFrom(Dll, Dll.Size);
        finally
          Free;
        end;
      finally
        Dll.Free;
      end;
      ShellExecute(Handle, 'open', PChar(FSettings.AddonPath + FPacks.InstallPack(
          OpenDialog.Filename, FSettings.AddonPath, DllPath, BrowseCallback)),
          '', '', SW_SHOW);
    finally
      DeleteFile(DllPath);
    end;
    ShowMessage('Addon pack successfully installed.');
  end;

  procedure InstallSoundfont;
  var
    Path, Name, Desc: String;
    I: Integer;
  begin
    Path:=OpenDialog.Filename;
    Name:=ExtractFileName(Path);
    Name:=Copy(Name, 1, Length(Name) - Length(ExtractFileExt(Name)));

    ///////GUARANTEE UNIQUE NAME.............../////////////////

    Desc:=Name;
    I:=2;
    while FPacks.SFPacks.IndexOfName(Name) > -1 do
    begin
      Name:=Desc + '_' + IntToStr(I);
      Inc(I);
    end;

    if InputQuery('Description',
      'Please enter a description for the soundfont:', Desc) then
        FPacks.InstallSoundfont(Path, Name, Desc);
    ShowMessage('Soundfont successfully installed.');
  end;

var
  Ext: String;
begin
  if FSettings.AddonPath = '' then
    with TPickFolder.Create(nil) do
    try
      Caption:='Please enter the folder where you want to store the contents of your packages:';
      Path:=FSettings.Path + 'Addon\Packages';
      if ShowModal = idCancel then
        Exit;
      FSettings.AddonPath:=Path;
      ForceDirectories(Path);
    finally
      Free;
    end;



  if not OpenDialog.Execute then
    Exit;

  WaitPanel.Caption:='Installing addon packs/soundfonts. Please wait...';
  WaitPanel.Visible:=True;
  try
    Application.ProcessMessages;
    Ext:=LowerCase(ExtractFileExt(OpenDialog.Filename));
    if Ext = '.ffaddon' then
    begin
      InstallPack;
      EnumPacks(True, True);
    end
    else if (Ext = '.sf2') or (Ext = '.dls') or (Ext = '.sfpack') then
    begin
      InstallSoundfont;
      EnumPacks(True, False);
    end;
  finally
    WaitPanel.Visible:=False;
  end;
end;

procedure TMainForm.RemovePack(Sender: TObject);
var
  Index,
  I    : Integer;
  Path : String;
  SGTs : TStringList;
begin
  if ActiveControl = Soundfonts then
  begin
    Index:=Soundfonts.ItemIndex;
    if Index < 0 then Exit;
    if Application.MessageBox(PChar('Are you sure you want to remove the soundfont "' +
      Soundfonts.Items[Index] + '" from the list of available soundfonts?'),
      PChar(Application.Title), MB_YESNO or MB_ICONQUESTION or MB_DEFBUTTON2) = idNo then
      Exit;

    Index:=Integer(Soundfonts.Items.Objects[Soundfonts.ItemIndex]);  
    Path:=FPacks.SFFile[Index];
    FPacks.DeleteSF(Index);
    EnumPacks(True, False);
    if Application.MessageBox(PChar('The soundfont is removed from the list. ' +
      'Do you also want to delete the soundfont file ("' + Path +
      '") from disk?'#10#10'If in doubt, answer "no".'),
      PChar(Application.Title), MB_YESNO or MB_ICONEXCLAMATION or MB_DEFBUTTON2) = idYes then
    begin
      DeleteFile(Path);
    end;
  end
  else if ActiveControl = MusicSets then
  begin
    Index:=MusicSets.ItemIndex;
    if Index < 0 then Exit;
    if Application.MessageBox(PChar('Are you sure you want to remove the music set "' +
      MusicSets.Items[Index] + '" from the list of available music sets?'),
      PChar(Application.Title), MB_YESNO or MB_ICONQUESTION or MB_DEFBUTTON2) = idNo then
      Exit;

    MusicSets.Items.Delete(Index); //Visual trick...

    if Application.MessageBox(PChar('The music set is removed from the list. ' +
      'Do you also want to delete the music files belonging ' +
      'to that set from disk?'),
      PChar(Application.Title), MB_YESNO or MB_ICONEXCLAMATION or MB_DEFBUTTON2) = idYes then
    begin
      SGTs:=TStringList.Create;
      try
        FPacks.GetSGTOverrides(SGTs, FPacks.SGTName[Index]);
        for I:=0 to SGTs.Count - 1 do
        begin
          Path:=SGTs.Values[SGTs.Names[I]];
          if Path[2] <> ':' then
            Path:=FSettings.AddonPath + Path;
          if Lowercase(ExtractFilePath(Path)) <> Lowercase(FPacks.DataPath) then  //We don't want to delete the files following the game
            DeleteFile(Path);
        end;
      finally
        SGTs.Free;
      end;
    end;
    FPacks.DeleteSGT(Index);
    EnumPacks(False, True);
  end;
end;

procedure TMainForm.LaunchURL(Sender: TObject);
var
  S: String;
begin
  case (Sender as TControl).Tag of
  1: S:='http://ffsf.cjb.net';
  2: S:='mailto:dagsverre@sensewave.com';
  3: S:='mailto:ivarkvaal@sensewave.com';
  end;
  TLauncherThread.Create(S);
end;

procedure TMainForm.MoveMusicSet(Direction: Integer);
var
  State1, State2: Boolean;
begin
  with MusicSets do
    if (ItemIndex + Direction > -1) and (ItemIndex + Direction < Items.Count) then
    begin
      State1:=Checked[ItemIndex];
      State2:=Checked[ItemIndex + Direction];
      Items.Exchange(ItemIndex, ItemIndex + Direction);
      Checked[ItemIndex]:=State2;
      Checked[ItemIndex + Direction]:=State1;
      ItemIndex:=ItemIndex + Direction;
    end;
end;

procedure TMainForm.MoveSetUp(Sender: TObject);
begin
  MoveMusicSet(-1);
end;

procedure TMainForm.MoveSetDown(Sender: TObject);
begin
  MoveMusicSet(1);
end;

procedure TMainForm.CheckHardware3D(Sender: TObject);
begin
  chkHardware.Enabled:=FDSDDraw.DeviceCaps[GraphicsDevices.ItemIndex].Hardware;
end;

procedure TMainForm.Install;
var
  CurPath,
  DestPath,
  DestExe: String;
begin
  CurPath:=ExtractFilePath(ParamStr(0));
  if FileExists(CurPath + 'ffaddon.dll') and FileExists(CurPath + 'playff8.exe') then
    Exit;

  try
    with TPickFolder.Create(nil) do
    try
      Caption:='Final Fantasy VIII Configurator is not installed. Please select '
        + 'an appropiate path for the program:';
      Path:=FSettings.Path + 'Addon';
      if ShowModal = idCancel then
        Exit;
      DestPath:=Path;
    finally
      Free;
    end;

    ForceDirectories(DestPath);
    ExtractResource('ffaddon_dll', DestPath + 'ffaddon.dll');
    ExtractResource('playff8_exe', DestPath + 'PlayFF8.exe');
    DestExe:=DestPath + 'FF8Configurator.exe';
    if CurPath <> DestPath then
      CopyFile(PChar(ParamStr(0)), PChar(DestExe), False);
    if Application.MessageBox(PChar('Program sucessfully installed. You should start the program by running '
         + DestExe + '". Do you want to create a shortcut to the program on your desktop?'),
         PChar(Application.Title), MB_YESNO or MB_ICONQUESTION) = idYes then
      CreateDesktopIcon(DestExe, 'Final Fantasy VIII Configurator.lnk');
    ShellExecute(Handle, 'open', PChar(DestExe), '', '', SW_SHOW);
  finally
    Halt;
  end;
end;

procedure TMainForm.Cancel(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ReadSettings;
var
  Ctrls : Boolean;

  procedure AssignButton(Control: TControl; Value: Integer);
  begin
    if (Value >= Low(ButtonNames)) and (Value <= High(ButtonNames)) and
      (Control is TEdit) then
    begin
      Control.Tag:=Value;
      TEdit(Control).Text:=ButtonNames[Control.Tag];
      if Value <> 0 then
        Ctrls:=True;
    end;
  end;

  procedure Split(List: TStrings; S: String; Delimiter: Char);
  var
    Point: Integer;
  begin
    while Length(S) > 0 do
    begin
      Point:=Pos(Delimiter, S);
      if Point = 0 then Point:=Length(S) + 1;
      List.Add(Copy(S, 1, Point - 1));
      Delete(S, 1, Point);
    end;
  end;

var
  I     : TButtonSettings;
  J     : Integer;
  Idx   : Integer;
  Names : TStringList;
begin
  chkHardware.Checked        :=     FSettings.BoolSettings[fbHardware];
  if chkHardware.Checked then
    RenderChange(chkHardware)
  else
    RenderChange(chkSoftware);
  chkQuarterScreen.Checked   :=     FSettings.BoolSettings[fbQuarterScreen];
  chkLoresVideo.Checked      := not FSettings.BoolSettings[fbHiResVideo];
  chkNoMapColor.Checked      := not FSettings.BoolSettings[fbWorldColor];
  chkNoMapTexture.Checked    := not FSettings.BoolSettings[fbWorldTexture];
  chkNoMapFog.Checked        := not FSettings.BoolSettings[fbWorldFog];
  chkNoBilinear.Checked      := not FSettings.BoolSettings[fbBilinear];
  chkLoresFonts.Checked      := not FSettings.BoolSettings[fbHiResFonts];
  chkReverseSpeakers.Checked :=     FSettings.BoolSettings[fbReverseSpeakers];
  chkEAX.Checked             :=     FSettings.BoolSettings[fbEAX];
  chkReverb.Checked          :=     FSettings.BoolSettings[fbReverb];
  chkChorus.Checked          :=     FSettings.BoolSettings[fbChorus];

  MusicDevices.ItemIndex     := FDSDMusic.IndexOfDevice(FSettings.GUIDSettings[fgMusic]);
  GraphicsDevices.ItemIndex  := FDSDDraw.IndexOfDevice(FSettings.GUIDSettings[fgGraphics]);
  SoundDevices.ItemIndex     := FDSDSound.IndexOfDevice(FSettings.GUIDSettings[fgSound]);

  if MusicDevices.ItemIndex < 0 then
    MusicDevices.ItemIndex:=0;
  if SoundDevices.ItemIndex < 0 then
    SoundDevices.ItemIndex:=0;
  if GraphicsDevices.ItemIndex < 0 then
    GraphicsDevices.ItemIndex:=0;

  Soundfonts.ItemIndex:=Soundfonts.Items.IndexOfObject(Pointer(
      FPacks.SFPacks.IndexOfName(FSettings.StringSettings[fsSoundfont])));
  if Soundfonts.ItemIndex < 0 then
    Soundfonts.ItemIndex:=0;

  Names:=TStringList.Create;
  try
    Split(Names, FSettings.StringSettings[fsMusicSets], SGTDelimiter);
    for J:=0 to MusicSets.Items.Count - 1 do
    begin
      Idx:=Names.IndexOf(FPacks.SGTName[J]);
      if Idx > -1 then
      begin
        MusicSets.Items.Exchange(Idx, MusicSets.Items.IndexOfObject(Pointer(J)));
        MusicSets.Checked[Idx]:=True;
      end;
    end;
  finally
    Names.Free;
  end;


  Ctrls:=False;
  for I:=Low(TButtonSettings) to High(TButtonSettings) do
  begin
    AssignButton(ControlsPage.FindChildControl('J' + IntToStr(Integer(I))),
      FSettings.JoystickSettings[I]);
    AssignButton(ControlsPage.FindChildControl('K' + IntToStr(Integer(I))),
      FSettings.KeyboardSettings[I]);
  end;
  if not Ctrls then
    LoadDefaultControls(nil);
end;

procedure TMainForm.WriteSettings(Sender: TObject);
  procedure SFConflict(const Error: String);
  begin
    Pages.ActivePage:=(Soundfonts.Parent as TTabSheet);
    Soundfonts.SetFocus;
    Application.MessageBox(PChar(Error), PChar(Application.Title), MB_ICONEXCLAMATION);
  end;

var
  I: Integer;
  S: String;
  Value: Boolean;
  DLS: Boolean;
  Soundfont,
  SoundfontType: String;
begin
  S:=ExtractFilePath(ParamStr(0));
  if not FileExists(S + 'config.ini') then
      if Application.MessageBox(PChar('Settings saved! Remember that you must start "'
      + S + 'PlayFF8.exe" instead of the original exe-file ("FF8.EXE") to play the game. '
      + 'Do you want to create a shortcut to this file on your desktop?'),
      PChar(Application.Title), MB_YESNO or MB_ICONQUESTION) = idYes then
    CreateDesktopIcon(S + 'PlayFF8.exe', 'Final Fantasy VIII Launcher.lnk');

  DLS:=FDSDMusic.DeviceCaps[MusicDevices.ItemIndex].dwEffectFlags and DMUS_PC_DLS = DMUS_PC_DLS;
  if Soundfonts.ItemIndex <= 0 then
    Soundfont:=''
  else
    Soundfont:=FPacks.SFFile[Integer(Soundfonts.Items.Objects[Soundfonts.ItemIndex])];
  SoundfontType:=Lowercase(ExtractFileExt(Soundfont));

  if (Soundfont <> '') and
     ((DLS and (SoundfontType <> '.dls')) or
      (not DLS and (SoundfontType <> '.sf2'))) then
  begin
    SFConflict('The soundfont you have selected is incompatible with '
      + 'the music device you have selected. Please change one of them. You could also try selecting no soundfonts: (default / manual loading).');
    Exit;
  end;

  if not DLS and (SoundfontType = '.sf2') then
  begin
    if Soundfont[2] <> ':' then
      Soundfont:=FSettings.Addonpath + Soundfont;
    SFConflict('Automatic loading of SF2 soundfonts is not currently supported. '
      + 'To use this soundfont on a Creative SoundBlaster Live!, follow these steps:'#10#10
      + '1. Select "(default / manual loading)" as your soundfont'#10
      + '2. Start AudioHQ and select soundfonts settings.'#10
      + '3. Load the soundfont ("' + Soundfont + '") into bank 0 (synth)'#10#10
      + 'If you can''t get it to work you should pay a visit to our official page: http://ffsf.cjb.net');
    Exit;
  end;

  FSettings.BoolSettings[fbHardware]        := chkHardware.Checked;
  FSettings.BoolSettings[fbQuarterScreen]   := chkQuarterScreen.Checked;
  FSettings.BoolSettings[fbHiResVideo]      := not chkLoresVideo.Checked;
  FSettings.BoolSettings[fbWorldColor]      := not chkNoMapColor.Checked;
  FSettings.BoolSettings[fbWorldTexture]    := not chkNoMapTexture.Checked;
  FSettings.BoolSettings[fbWorldFog]        := not chkNoMapFog.Checked;
  if chkNoBilinear.Enabled then Value:=chkNoBilinear.Checked else Value:=True;
  FSettings.BoolSettings[fbBilinear]        := not Value;
  if chkLoresFonts.Enabled then Value:=chkLoresFonts.Checked else Value:=True;
  FSettings.BoolSettings[fbHiResFonts]      := not Value;
  FSettings.BoolSettings[fbReverseSpeakers] := chkReverseSpeakers.Checked;
  FSettings.BoolSettings[fbEAX]             := chkEAX.Checked;
  FSettings.BoolSettings[fbReverb]          := chkReverb.Checked;
  FSettings.BoolSettings[fbChorus]          := chkChorus.Checked;
  FSettings.BoolSettings[fbXG]              := (FDSDMusic.DeviceCaps[MusicDevices.ItemIndex].dwFlags
                                              or DMUS_PC_XGINHARDWARE) = DMUS_PC_XGINHARDWARE;

  FSettings.GUIDSettings[fgMusic]    := FDSDMusic.DeviceCaps[MusicDevices.ItemIndex].guidPort;
  FSettings.GUIDSettings[fgSound]    := FDSDSound.DeviceID[SoundDevices.ItemIndex];
  FSettings.GUIDSettings[fgGraphics] := FDSDDraw.DeviceCaps[GraphicsDevices.ItemIndex].DeviceID;


  if Soundfonts.ItemIndex <= 0 then
    Soundfont:=''
  else
    Soundfont:=FPacks.SFName[Integer(Soundfonts.Items.Objects[Soundfonts.ItemIndex])];
  FSettings.StringSettings[fsSoundfont] := Soundfont;

  S:='';
  for I:=0 to MusicSets.Items.Count - 1 do
    if MusicSets.Checked[I] then
      S:=S + FPacks.SGTName[Integer(MusicSets.Items.Objects[I])] + SGTDelimiter;
  FSettings.StringSettings[fsMusicSets] := S;

  with ControlsPage do
    for I:=0 to ControlCount - 1 do
    begin
      S:=Controls[I].Name;
      if Length(S) > 0 then
        if Controls[I].Name[1] = 'J' then
        begin
          Delete(S, 1, 1);
          FSettings.JoystickSettings[TButtonSettings(StrToInt(S))]:=Controls[I].Tag
        end
        else if Controls[I].Name[1] = 'K' then
        begin
          Delete(S, 1, 1);
          FSettings.KeyboardSettings[TButtonSettings(StrToInt(S))]:=Controls[I].Tag
        end;
    end;

  Close;
end;

procedure TMainForm.RenderChange(Sender: TObject);
begin
  chkNoBilinear.Enabled:=Sender = chkHardware;
  chkNoBilinear.Checked:=Sender = chkSoftware;
end;

procedure TMainForm.NullifyKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Key:=0;
end;

procedure TMainForm.EnumPacks(SF, SGT: Boolean);
var
  Sel,
  I: Integer;
begin
  FPacks.Flush;
  if SF then
  begin
    Sel:=Soundfonts.ItemIndex;
    Soundfonts.Clear;
    Soundfonts.Items.AddObject('(default / manual loading)', Pointer(-2)); //-1 isn't added as an object for some reason...maybe used internal in StringList?
    for I:=0 to FPacks.SFPacks.Count - 1 do
      Soundfonts.Items.AddObject(FPacks.SFDescription[I], Pointer(I));
    if (Sel < 0) or (Sel > Soundfonts.Items.Count - 1) then
      Sel:=0;
    Soundfonts.ItemIndex:=Sel;
  end;
  if SGT then
  begin
    Sel:=MusicSets.ItemIndex;
    MusicSets.Clear;
    for I:=0 to FPacks.SGTPacks.Count - 1 do
      MusicSets.Items.AddObject(FPacks.SGTDescription[I], Pointer(I));
    if (Sel < 0) or (Sel > MusicSets.Items.Count - 1) then
      Sel:=0;
    MusicSets.ItemIndex:=Sel;
  end;
end;

procedure TMainForm.ExtractResource(const ResName, DestPath: String);
var
  Fil: TFileStream;
  Res: TResourceStream;
begin
  Fil:=nil; Res:=nil;
  try
    Fil:=TFileStream.Create(DestPath, fmCreate);
    Res:=TResourceStream.Create(hInstance, ResName, RT_RCDATA);
    Fil.CopyFrom(Res, Res.Size);
  finally
    Fil.Free;
    Res.Free;
  end;
end;


procedure TMainForm.LoadDefaultControls(Sender: TObject);
  procedure AssignButton(Control: TControl; Value: Integer);
  begin
    if (Value >= Low(ButtonNames)) and (Value <= High(ButtonNames)) and
      (Control is TEdit) then
    begin
      Control.Tag:=Value;
      TEdit(Control).Text:=ButtonNames[Control.Tag];
    end;
  end;

const
  DefaultKeys: array[TButtonSettings] of Integer =
  (32, 45, 30, 17, 16, 18, 44, 46, 31, 33, 200, 208, 203, 205);


  DefaultJoys: array[TButtonSettings] of Integer =
  (226, 225, 224, 227, 228, 229, 230, 231, 232, 233, 252, 253, 254, 255);
var
  I: TButtonSettings;
begin
  for I:=Low(TButtonSettings) to High(TButtonSettings) do
  begin
    AssignButton(ControlsPage.FindChildControl('J' + IntToStr(Integer(I))),
      DefaultJoys[I]);
    AssignButton(ControlsPage.FindChildControl('K' + IntToStr(Integer(I))),
      DefaultKeys[I]);
  end;
end;

procedure TMainForm.chkNoBilinearClick(Sender: TObject);
begin
  NoBilinear.Visible:=chkNoBilinear.Checked;
end;

procedure TMainForm.PackEnter(Sender: TObject);
begin
  RemoveButton.Enabled:=(Sender = Musicsets) or (Soundfonts.ItemIndex <> 0);
end;

procedure TMainForm.PackExit(Sender: TObject);
begin
  RemoveButton.Enabled:=False;
end;

function TMainForm.BrowseCallBack(var AFilename: String): Boolean;
begin
  Application.MessageBox('Couldn''t find the file to update or the file didn''t '
    + 'match with the file the update is made for. Please select it manually.',
    PChar(Application.Title), MB_ICONEXCLAMATION);
  with TOpenDialog.Create(Self) do
  try
    Title:='Please find "' + ExtractFileName(AFileName) + '"...';
    Filename:=AFilename;
    Filter:=ExtractFileName(AFilename);
    Filter:=Filter + '|' + Filter + '|All files|*.*';
    Result:=Execute;
    AFilename:=Filename;
  finally
    Free;
  end;
end;

procedure TMainForm.CreateDesktopIcon(const Path, DestFile: String);
var
  ShellLink: IShellLink;
  PersistFile: IPersistFile;
  DesktopPath: WideString;
begin
  with TRegistry.Create do
  try
    RootKey:=HKEY_CURRENT_USER;
    if not OpenKey('\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False) then
      raise Exception.Create('Cannot find desktop path.');
    DesktopPath:=ReadString('Desktop');
  finally
    Free;
  end;
  if DesktopPath[Length(DesktopPath)] <> '\' then
    DesktopPath:=DesktopPath + '\';
  DesktopPath:=DesktopPath + DestFile;

  OleCheck(CoCreateInstance(CLSID_ShellLink, nil, CLSCTX_INPROC_SERVER,
                            IShellLink, ShellLink));
  PersistFile:=ShellLink as IPersistFile;
  OleCheck(ShellLink.SetPath(PChar(Path)));
  OleCheck(PersistFile.Save(PWideChar(DesktopPath), True));
end;

initialization

  RegisterClasses([TSpeedButton, TPageControl, TBevel, TTimer,
                   TPanel, TShape, TLabel, TEdit]);
end.
