unit shortcut;

interface
uses SysUtils, Windows, Registry, OLE2,comctrls;
const
  sfDesktop   = 'Desktop';
  sfFavorites = 'Favorites';
  sfFonts     = 'Fonts';
  sfNetHood   = 'NetHood';
  sfPersonal  = 'Personal';
  sfPrograms  = 'Programs';
  sfRecent    = 'Recent';
  sfSendTo    = 'SendTo';
  sfStartMenu = 'Start Menu';
  sfStartup   = 'Startup';
  sfTemplates = 'Templates';

  HOTKEYF_SHIFT           = $01;
  HOTKEYF_CONTROL         = $02;
  HOTKEYF_ALT             = $04;
  HOTKEYF_EXT             = $08;
type
  EShellOleError = class(Exception);

  THotKeyRec = record
    Char, ModCode: Byte;
  end;

   TShortcut=class(TObject)
   private
       FDesc,
       FPathName,
       FArg,
       fWorkDir,
       fIconFile:string;
       fShowcmd,
       fIconIndex:integer;
       fHotkey:word;
       function GetFolderLocation(const FolderType: String): String;
   public
       procedure SaveToFile(filename:string);
       property ShellFolders[const FolderType:string]:string read GetFolderLocation;
       property PathName:string read FPathName write fPathname;
       property Description:string read FDesc write FDesc;
       property Arguments:string read fArg write fArg;
       property WorkingDirectory:string read fWorkdir write fWorkdir;
       property IconFile:string read fIconFile write fIconFile;
       property IconIndex:integer read fIconIndex write fIconIndex;
       property ShowCmd:integer read fShowcmd write fShowcmd;
       property Hotkey:word read fHotKey write fHotKey;

   end;
implementation
uses
   shlObj{,strutils};
const
  { Registry key where Folder information is kept }
  SFolderKey = '\Software\Microsoft\Windows\CurrentVersion\' +
               'Explorer\Shell Folders';

function HotKeyToWord(HotKey:word;CtlMods:THKModifiers):word;
var
  HR: THotKeyRec;
begin
    word(HR) := HotKey;
    with HR do begin
      ModCode := 0;
      if (hkAlt in CtlMods) then ModCode := ModCode or HOTKEYF_ALT;
      if (hkCtrl in CtlMods) then ModCode := ModCode or HOTKEYF_CONTROL;
      if (hkExt in CtlMods) then ModCode := ModCode or HOTKEYF_EXT;
      if (hkShift in CtlMods) then ModCode := ModCode or HOTKEYF_SHIFT;
    end;
    result := word(HR);
end;

procedure OLECheck(OleRetVal: HResult);
{ Checks the HResult return value of an OLE function.  Raises an }
{ exception if value is something other than S_OK. }
const
  OleErrStr = 'OLE function call failed. HResult is $%x. GetLastError is $%x';
begin
  if OleRetVal <> S_OK then
    raise EShellOleError.CreateFmt(OleErrStr, [OleRetVal, GetLastError]);
end;

procedure TShortCut.SaveToFile(filename:string);
var
  SL: IShellLink;
  PF: IPersistFile;
//  LnkName: String;
  WStr: array[0..MAX_PATH - 1] of WideChar;
begin
//  if copy(filename,length(filename)-3,4) <> '.lnk' then
//       ChangeFileExt(filename, 'lnk');
  OleCheck(CoCreateInstance(shlObj.CLSID_ShellLink, Nil, CLSCTX_INPROC_SERVER,
                            IID_IShellLink, SL));
  try
    OleCheck(SL.QueryInterface(IID_IPersistFile, PF));
    try
      with SL do
      begin
        OleCheck(SetPath(PChar(PathName)));
        OleCheck(SetArguments(PChar(Arguments)));
        OleCheck(SetDescription(PChar(Description)));
        OleCheck(SetWorkingDirectory(PChar(WorkingDirectory)));
        OleCheck(SetIconLocation(PChar(IconFile), IconIndex));
        OleCheck(SetShowCmd(ShowCmd));
        OleCheck(SetHotKey(HotKey));
      end;
      StringToWideChar(Filename, WStr, MAX_PATH);
      PF.Save(WStr, True);                   // save link file
    finally
      PF.Release;
    end;
  finally
    SL.Release;
  end;
//  Result := LnkName;
end;

function TShortcut.GetFolderLocation(const FolderType: String): String;
{ Retrieves from registry path to folder indicated in FolderType }
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    with Reg do begin
      RootKey := HKEY_CURRENT_USER;
      if not OpenKey(SFolderKey, False) then
        { open key where shell folder information is kept. }
        raise ERegistryException.CreateFmt('Folder key "%s" not found',
                                           [SFolderKey]);
      { Get path for specified folder }
      Result := ReadString(FolderType);
      if Result = '' then
        raise ERegistryException.CreateFmt('"%s" item not found in registry',
                                           [FolderType]);
      if result[length(result)] <> '\' then
           result:=result+'\';
      CloseKey;
    end;
  finally
    Reg.Free;
  end;
end;


initialization
  OleInitialize(Nil);
finalization
  OleUninitialize;

end.
