unit main;
{*************TMainForm************
Main unit of PlayFF8.EXE
This program manages tuning of the registry before playing,
cleaning up afterwards, loading/unloading of soundfonts etc.

Made in Delphi 4

Author:  Dag Sverre Seljebotn, dagsverre@sensewave.com
Created: 14-04-2000
History: 20-03-2000 First release

I used the UPX packer...

Distributed together with FF8Configurator.
http://sfff.cjb.net

This code is distributed under the GNU General Public License.
NO WARRANTY. See COPYING for details.

Copyright (c) 2000 Dag Sverre Seljebotn
}

interface

uses Windows, SysUtils, ActiveX, ComObj;

const
  psStart = 'start';
  psCleanup = 'cleanup';

{$L sfwrap}

//function SFLoadBank(BankIndex: Integer): Integer; external;



implementation
uses Config, Registry, Splash {, SFPACKLib_TLB, sfman};

const
  CLSID_DirectMusicPerformance : TGUID = '{d2ac2881-b39b-11d1-8704-00600893b1bd}';
  CLSID_DirectMusicLoader : TGUID = '{d2ac2892-b39b-11d1-8704-00600893b1bd}';

  RunOnceKey = '\Software\Microsoft\Windows\CurrentVersion\RunOnce';
  RunOnceValue = 'FF8 Configurator cleanup';

(*procedure LoadSoundfont(Path: String);
  procedure LoadSF2;
  begin
//    SFLoadBank(2);
  end;

var
  Engine: IEngine;
  SFPack: ISFPackFile;
  Ext: String;

begin
  Ext:=Lowercase(ExtractFileExt(Path));
  if Ext = '.sfpack' then
  begin
    Path:=Copy(Path, 1, Length(Path) - Length(Ext));
    if not FileExists(Path + '.sf2') then
    begin
      CoInitialize(nil);
      if Failed(CoCreateInstance(CLASS_ENGINE, nil, CLSCTX_LOCAL_SERVER,
       IEngine, Engine)) then
      begin
        MessageBox(0, 'Megota SFPack is not installed. Get it at http://www.megota.com, or select another soundfont.', 'Can''t load soundfont', MB_OK);
        Exit;
      end;
      SFPack:=Engine.OpenSFPack(Path + '.sfpack');
      SFPack.Inflate(Path + '.sf2');
    end
    else
      Path:=ChangeFileExt(Path, '.sf2');
  end;

  if Ext = '.sf2' then
    LoadSF2;

  //We elegantly ignore DLS....
end;
*)
procedure InstallRunOnce(Install: Boolean);
begin
  with TRegistry.Create do
  try
    RootKey:=HKEY_LOCAL_MACHINE;
    LazyWrite:=False;
    if OpenKey(RunOnceKey, True) then
      if Install then
        WriteString(RunOnceValue, ParamStr(0) + ' /cleanup')
      else
        DeleteValue(RunOnceValue);
  finally
    Free;
  end;
end;

procedure StartProcessAndWait(const Path, StartingDir: String);
var
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
begin
  FillChar(StartupInfo, SizeOf(StartupInfo), 0);
  with StartupInfo do
  begin
    cb:=SizeOf(StartupInfo);
    dwFlags:=STARTF_FORCEONFEEDBACK;
  end;
  if not CreateProcess(nil, PChar(Path), nil, nil, False, NORMAL_PRIORITY_CLASS, nil,
    PChar(StartingDir), StartupInfo, ProcessInfo) then
      raise Exception.Create('Couldn''t load Final Fantasy VIII. Check that the game is correctly installed.');
  WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
  CloseHandle(ProcessInfo.hThread);
  CloseHandle(ProcessInfo.hProcess);
end;

procedure Messup;
var
  DllPath: String;
begin
  InstallRunOnce(True);
  DllPath:=ExtractFilePath(ParamStr(0));
  with TFF8ConfigIni.Create(DllPath + 'config.ini') do
  try
    DllPath:=DllPath + 'ffaddon.dll';
    with TRegistry.Create do
    try
      RootKey:=HKEY_CLASSES_ROOT;
      LazyWrite:=False;
      if OpenKey('CLSID\' + GUIDToString(CLSID_DirectMusicLoader) + '\InprocServer32', True) then
        if ReadString('') <> DllPath then
        begin
          BackupCLSIDs([CLSID_DirectMusicLoader]);
          WriteString('', DllPath);
        end;
    finally
      Free;
    end;
    ActivateSettings;
    DllPath:=Path;
//    LoadSoundfont(StringSettings[fsSoundfont]);
  finally
    Free;
  end;
  StartProcessAndWait(DllPath + 'FF8.EXE', DllPath);
end;

procedure Cleanup;
begin
  with TFF8ConfigIni.Create(ExtractFilePath(ParamStr(0)) + 'config.ini') do
  try
    RestoreCLSIDs;
    DeActivateSettings;
    InstallRunOnce(False);
  finally
    Free;
  end;
end;

procedure MainProc;
  function Param(S: String): Boolean;
  begin
    Result:=Pos(S, LowerCase(ParamStr(1))) > 0;
  end;
begin
  if (ParamCount < 1) or Param(psStart) then
  try
    SplashScreen.Show;
    Messup;
  finally
    Cleanup;
  end
  else if Param(psCleanup) then
    Cleanup;
end;
initialization
  try
    MainProc;
  except
    on E: Exception do MessageBox(0, PChar(
      'An error have occurred. Try running FF8.EXE instead.'#10#10 +
      E.ClassName + ': ' + E.Message), 'Error', MB_OK or MB_ICONEXCLAMATION);
  end;
end.
