unit splash;

interface
uses Windows, Messages, SysUtils, Classes, Graphics, JPEG;

type
  TSplashScreen = class
  private
    FHandle  : THandle;
    FImage   : TGraphic;

    procedure Paint;
    procedure LoadImage;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Show;

    property Handle: THandle read FHandle;
  end;

var
  SplashScreen: TSplashScreen;

implementation

const
  Height = 265;
  Width  = 330;
  FFCaption = 'Loading Final Fantasy VIII. Please wait...';
{ TSplashScreen }

{$R splash\splash.res}

function WinProc(HWnd, Msg, WParam, LParam: Integer): Integer; stdcall;
begin
  Result:=0;
  case Msg of
  WM_DESTROY:
    PostQuitMessage(0);
  WM_PAINT:
    SplashScreen.Paint;
  else
    Result:=DefWindowProc(HWnd,Msg,WParam,LParam);
  end;
end;

constructor TSplashScreen.Create;
  procedure RegisterClass;
  var
    WindowClass:TWndClassEx;
  begin
    WindowClass.cbSize:=SizeOf(WindowClass);
    WindowClass.Style:=cs_hRedraw or cs_vRedraw;
    WindowClass.lpfnWndProc:=@WinProc;
    WindowClass.cbClsExtra:=0;
    WindowClass.cbWndExtra:=0;
    WindowClass.hInstance:=hInstance;
    WindowClass.hIcon:=LoadIcon(0,idi_Application);
    WindowClass.hIconSm:=LoadIcon(0,idi_Application);
    WindowClass.hCursor:=LoadCursor(0,idc_Arrow);
    WindowClass.hbrBackground:=HBrush(Color_Window);
    WindowClass.lpszMenuName:=nil;
    WindowClass.lpszClassName:=PChar(String(ClassName));

    if Windows.RegisterClassEx(WindowClass) = 0 then
      raise Exception.Create('Couldn''t register windowclass');
  end;

  procedure CreateWindow;
  begin
    FHandle:=Windows.CreateWindow(PChar(String(ClassName)), PChar(''),
              WS_POPUP, (GetSystemMetrics(SM_CXSCREEN) - Width) div 2,
              (GetSystemMetrics(SM_CYSCREEN) - Height - 100) div 2,
              Width, Height, 0, 0, HInstance, nil);
  end;
begin
  inherited Create;
  RegisterClass;
  CreateWindow;
  LoadImage;
end;


destructor TSplashScreen.Destroy;
begin
  DestroyWindow(FHandle);
  inherited Destroy;
end;

procedure TSplashScreen.LoadImage;
var
  ImageFile : TStream;
begin
  FImage := TJPEGImage.Create; //OK, not very good, but it allows for GIFs/BMPs later
  Randomize;
  ImageFile := TResourceStream.Create(hInstance, 'Image' + IntToStr(Random(5)), RT_RCDATA);
  try
    FImage.LoadFromStream(ImageFile);
  finally
    ImageFile.Free;
  end;
end;

procedure TSplashScreen.Paint;
var
  Canvas      : TCanvas;
  PaintStruct : TPaintStruct;
  DC          : Cardinal;
begin
  DC:=BeginPaint(FHandle, PaintStruct);
  try
    Canvas:=TCanvas.Create;
    with Canvas do
    try
      Handle      := DC;
      Pen.Width   := 3;
      Pen.Color   := clBlack;
      Polygon([Point(1, 1), Point(Width - 2, 1), Point(Width - 2, Height - 2),
        Point(1, Height - 2)]);

      Draw(5, 5, FImage);

      Font.Name   := 'Tahoma';
      Brush.Style := bsClear;
      Font.Style  := [fsBold];
      Canvas.TextOut((Width - TextWidth(FFCaption)) div 2, Height - 28, FFCaption);

    finally
      Canvas.Free;
    end;
  finally
    EndPaint(DC, PaintStruct);
  end;
end;

procedure TSplashScreen.Show;
begin
  ShowWindow(FHandle, SW_SHOW);
  UpdateWindow(Handle);
end;

initialization
  SplashScreen:=TSplashScreen.Create;

finalization
  SplashScreen.Free;

end.