unit packs;
///NOT THREAD-SAFE coz o' rar...
interface
uses Classes, FF8Ini;


const
  SF = 0;
  SGT = 1;



type
  TBrowseFunc = function (var Path: String): Boolean of object;

  TPackManager = class(TFF8IniFile)
  private
    FSoundfonts: TStringList;
    FSGTs: TStringList;

    function GetPack(Index: Integer): TStrings;
    function GetDesc(I, Index: Integer): String;
    function GetFile(I, Index: Integer): String;
    function GetName(I, Index: Integer): String;

  protected
    procedure WriteSectionValues(const Section: String; Values: TStrings);
  public
    destructor Destroy; override;

    function InstallPack(const Filename, DestPath, UnRarDll: String; BrowseFunc: TBrowseFunc): String;
    procedure InstallSoundfont(const Filename, Name, Description: String);

    procedure DeleteSF(Index: Integer);
    procedure DeleteSGT(Index: Integer);

    procedure GetSGTOverrides(List: TStrings; SGTPacks: String);

    procedure Flush;

    property SFPacks: TStrings index SF read GetPack;
    property SFName[I: Integer]: String index SF read GetName;
    property SFFile[I: Integer]: String index SF read GetFile;
    property SFDescription[I: Integer]: String index SF read GetDesc;

    property SGTPacks: TStrings index SGT read GetPack;
    property SGTName[I: Integer]: String index SGT read GetName;
    property SGTFile[I: Integer]: String index SGT read GetFile;
    property SGTDescription[I: Integer]: String index SGT read GetDesc;
  end;


implementation
uses Windows, SysUtils, Rar, Inifiles, aDiff;

const
  PackIni         = 'package.ini';  //Filename

  SoundfontsSection = 'Soundfonts'; //[Soundfonts]
  SGTsSection = 'Musicsets';

procedure TPackManager.DeleteSF(Index: Integer);
var
  Name: String;
begin
  Name:=SFName[Index];
  EraseSection(Name);
  DeleteKey(SoundfontsSection, Name);
end;

procedure TPackManager.DeleteSGT(Index: Integer);
var
  Name: String;
begin
  Name:=SGTName[Index];
  EraseSection(Name);
  DeleteKey(SGTsSection, Name);
end;

destructor TPackManager.Destroy;
begin
  Flush;
  inherited Destroy;
end;

procedure TPackManager.Flush;
begin
  FSoundfonts.Free; FSoundfonts:=nil;
  FSGTs.Free; FSGTs:=nil;
end;

function TPackManager.GetDesc(I, Index: Integer): String;
begin
  if I < 0 then
    Result:=''
  else
    case Index of
    SF:  Result:=SFPacks.Values[SFName[I]];
    else Result:=SGTPacks.Values[SGTName[I]];
    end;
end;

function TPackManager.GetFile(I, Index: Integer): String;
begin
  if I < 0 then
    Result:=''
  else
    case Index of
    SF: Result:=ReadString(SFName[I], 'Path', '');//No entry for default
    end;
end;

function TPackManager.GetName(I, Index: Integer): String;
begin
  if I < 0 then
    Result:=''
  else
    case Index of
    SF:  Result:=SFPacks.Names[I];
    else Result:=SGTPacks.Names[I];
    end;
end;

function TPackManager.GetPack(Index: Integer): TStrings;
begin
  if Index = SF then
  begin
    if FSoundfonts = nil then
    begin
      FSoundfonts:=TStringList.Create;
      ReadSectionValues(SoundfontsSection, FSoundfonts);
    end;
    Result:=FSoundfonts;
  end
  else
  begin
    if FSGTs = nil then
    begin
      FSGTs:=TStringList.Create;
      ReadSectionValues(SGTsSection, FSGTs);
    end;
    Result:=FSGTs;
  end;
end;

procedure TPackManager.GetSGTOverrides(List: TStrings; SGTPacks: String);
var
  Section: TStringList;
  P, I: Integer;
begin
  Section:=TStringList.Create;
  try
    while Length(SGTPacks) > 0 do
    begin
      P:=Pos('|', SGTPacks);
      if P = 0 then P:=Length(SGTPacks) + 1;
      Section.Clear;
      ReadSectionValues(Copy(SGTPacks, 1, P - 1), Section);
      Delete(SGTPacks, 1, P);
      for I:=0 to Section.Count - 1 do
        if List.IndexOfName(Section.Names[I]) <> -1 then
          List[I]:=List[I] + '|' + Section.Values[Section.Names[I]]
        else
          List.Add(Section[I]);
    end;
  finally
    Section.Free;
  end;
end;

function TPackManager.InstallPack(const Filename, DestPath, UnRarDll: String; BrowseFunc: TBrowseFunc): String;
  procedure MergeIni(const Filename: String);
  var
    Ini: TIniFile;
    Sections,
    Section: TStringList;
    I: Integer;
  begin
    Ini:=nil; Sections:=nil; Section:=nil;
    try
      Ini:=TIniFile.Create(Filename);
      Sections:=TStringList.Create;
      Section:=TStringList.Create;

      Ini.ReadSections(Sections);
      for I:=0 to Sections.Count - 1 do
      begin
        Section.Clear;
        Ini.ReadSectionValues(Sections[I], Section);
        WriteSectionValues(Sections[I], Section);
      end;
    finally
      Section.Free;
      Sections.Free;
      Ini.Free;
    end;
  end;

  function Patch(Source, Dest, Diff: String): Boolean;
  var
    SourceStream,
    DiffStream,
    DestStream: TStream;
  begin
    Result:=True;
    if (Dest = '') or (not FileExists(Source)) or (not FileExists(Diff)) then
      raise Exception.Create('Cannot update soundfont. Please check that you have the old version of the soundfont before updating.');
    SourceStream:=nil; DestStream:=nil; DiffStream:=nil;
    try
      SourceStream:=TFileStream.Create(Source, fmOpenRead or fmShareDenyNone);
      DestStream  :=TFileStream.Create(Dest, fmCreate);
      DiffStream  :=TFileStream.Create(Diff, fmOpenRead or fmShareDenyNone);
      try
        aDiff.DiffStreamExtract(DiffStream, SourceStream, DestStream, nil);
      except
        on CRCError do Result:=False
        else raise;
      end;
    finally
      SourceStream.Free; DestStream.Free; DiffStream.Free;
    end;
  end;

const
  DiffSection = 'Diff';
  SetupIniName = 'setup.ini';
var
  DestFile,
  DiffFile,
  SourceFile,
  TempFile: String;

  Diffs: TStringList;
  I, J: Integer;
  KeepOld: Boolean;
  CRCPassed: Boolean;
begin
  if Rar.LoadDll(UnRarDll) then
  try
    Rar.ExtractRar(Filename, DestPath, [''], nil, nil, nil, '');
  finally
    Rar.FreeDll;
  end;
  MergeIni(DestPath + PackIni);
  DeleteFile(DestPath + PackIni);

  Diffs:=nil;
  with TIniFile.Create(DestPath + SetupIniName) do
  try
    Diffs:=TStringList.Create;
    Result:=ReadString('Options', 'Readme', '');
    ReadSection(DiffSection, Diffs);
    for I:=0 to Diffs.Count - 1 do
    begin


      ////////////////////

      DestFile   := DestPath + ReadString(Diffs[I], 'DestFile', '');
      DiffFile   := DestPath + ReadString(Diffs[I], 'DiffFile', '');
      SourceFile := DestPath + ReadString(Diffs[I], 'SourceFile', '');
      KeepOld    := not ReadBool(Diffs[I], 'DeleteOld', True);
      aDiff.BufSize:=ReadInteger(Diffs[I], 'Buffersize', 4096) * 1024;

      DeleteFile(DestPath + SetupIniName);
      repeat
        if not FileExists(SourceFile) then
          if not BrowseFunc(SourceFile) then
            raise Exception.Create('Package updating aborted!');
        if Lowercase(SourceFile) = Lowercase(DestFile) then
        begin
          J:=0;
          repeat
            TempFile:=ExtractFileExt(SourceFile);
            TempFile:=Format('%s_backup%d%s', [Copy(SourceFile, 1,
              Length(SourceFile) - Length(TempFile)), J, TempFile]);
            J:=J + 1;
          until not FileExists(TempFile);
          KeepOld:=False;
          MoveFile(PChar(SourceFile), PChar(TempFile));
          SourceFile:=TempFile;
        end;
        CRCPassed:=Patch(SourceFile, DestFile, DiffFile); //False = CRC error
        if not CRCPassed then
          SourceFile:='';
      until CRCPassed;
      DeleteFile(DiffFile);
      if not KeepOld and FileExists(DestFile) then
        DeleteFile(SourceFile);

      //////////////////////////////

    end;
  finally
    Diffs.Free;
    Free;
  end;
end;

procedure TPackManager.InstallSoundfont(const Filename, Name,
  Description: String);
begin
  WriteString(SoundfontsSection, Name, Description);
  WriteString(Name, 'Path', Filename);
end;

procedure TPackManager.WriteSectionValues(const Section: String;
  Values: TStrings);
var
  I: Integer;
  S: String;
begin
  for I:=0 to Values.Count - 1 do
  begin
    S:=Values.Names[I];
    WriteString(Section, S, Values.Values[S]);
  end;
end;

end.
