unit ILoader;

{Fake IDirectMusicLoader object. Or at least the beginning of it.
 Created (or will be sometime I feel for it) the mean, effective, hard way...}

interface
uses Windows, ActiveX, Classes, DirectMusic;

  function DllGetClassObject(const CLSID, IID: TGUID; var Obj): HResult; stdcall;
  function DllCanUnloadNow: HResult; stdcall;

type
  TClassFactory = class(TObject, IUnknown, IClassFactory)
  private
    FClassID: TGUID;
    FLocks: Integer;
  protected
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    function CreateInstance(const unkOuter: IUnknown; const iid: TIID;
      out obj): HResult; stdcall;
    function LockServer(Lock: BOOL): HResult; stdcall;
  public
    constructor Create(const ClassID: TGUID);
    property Locks: Integer read FLocks;
    property ClassID: TGUID read FClassID;
  end;

  TComServer = class
  private
    FList: TList;
    FObjectCount: Integer;
    function GetFactory(ClassID: TGUID): TClassFactory;
  public
    constructor Create;
    destructor Destroy; override;
    procedure ObjectCount(Add: Boolean);
    procedure AddFactory(Factory: TClassFactory);
    property Factory[ClassID: TGUID]: TClassFactory read GetFactory;
  end;

var
  ComServer: TComServer = nil;

implementation
const
//  Class_IDMLoader: TGUID = '{F6965061-0E47-11D4-8B12-706472C10000}';
  Class_IDMLoader: TGUID = '{D1AC2892-B39B-11D1-8704-00600893B1BD}';

function IDLGetClassObject(const CLSID, IID: TGUID; var Obj): HResult; stdcall; external 'dmloader.dll' name 'DllGetClassObject';

function DllGetClassObject(const CLSID, IID: TGUID; var Obj): HResult; stdcall;
var
  Factory: IClassFactory;
begin
  try
    Factory:=ComServer.Factory[CLSID];
    if Factory = nil then
      Result:=E_FAIL
    else if IsEqualIID(IID, IClassFactory) then
      Result:=Factory.QueryInterface(IID, Obj)
    else
      Result := E_NOINTERFACE;
  except
    Result:=E_OUTOFMEMORY;
  end;
end;

function DllCanUnloadNow: HResult; stdcall;
begin
  if ComServer.FObjectCount <= 0 then
    Result:=S_OK
  else
    Result:=S_FALSE;
end;

{ TClassFactory }

constructor TClassFactory.Create(const ClassID: TGUID);
begin
  inherited Create;
  FClassID:=ClassID;
  ComServer.AddFactory(Self);
end;

function TClassFactory.CreateInstance(const unkOuter: IUnknown;
  const iid: TIID; out obj): HResult;
var
  DMLoaderFactory: IClassFactory;
begin
  Pointer(obj):=nil;
  try
    if (unkOuter <> nil) then
      Result:=E_FAIL
    else
    begin
      if DMLoaderFactory = nil then
      begin
        Result:=IDLGetClassObject(CLSID_DirectMusicLoader, IClassFactory, DMLoaderFactory);
        if Failed(Result) then
          Exit;
      end;
      Result:=DMLoaderFactory.CreateInstance(nil, iid, obj);
      DMLoaderFactory:=nil;
    end;
  except
    Result:=E_OUTOFMEMORY;
  end;
end;

function TClassFactory.LockServer(Lock: BOOL): HResult;
begin
  if Lock then
    Inc(FLocks)
  else
    Dec(FLocks);
  Result:=S_OK;
end;

function TClassFactory._AddRef: Integer;
begin
  ComServer.ObjectCount(True);
end;

function TClassFactory._Release: Integer;
begin
  ComServer.ObjectCount(False);
end;

function TClassFactory.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if GetInterface(IID, Obj) then
    Result:=S_OK
  else
    Result:=E_NOINTERFACE;
end;

{ TClassFactoryList }

procedure TComServer.AddFactory(Factory: TClassFactory);
begin
  FList.Add(Factory);
end;

constructor TComServer.Create;
begin
  inherited Create;
  FList:=TList.Create;
end;

destructor TComServer.Destroy;
var
  I: Integer;
begin
  for I:=0 to FList.Count - 1 do
    TClassFactory(FList[I]).Free;
  FList.Free;
  inherited Destroy;
end;

procedure TComServer.ObjectCount(Add: Boolean);
begin
  if Add then
    Inc(FObjectCount)
  else
    Dec(FObjectCount);
end;

function TComServer.GetFactory(ClassID: TGUID): TClassFactory;
var
  I: Integer;
begin
  Result:=nil;
  for I:=0 to FList.Count - 1 do
    if IsEqualIID(TClassFactory(FList[I]).ClassID, ClassID) then
    begin
      Result:=TClassFactory(FList[I]);
      Break;
    end;
end;

initialization
  ComServer:=TComServer.Create;

  TClassFactory.Create(Class_IDMLoader);

finalization
  ComServer.Free;
end.
