unit iDMLoader;

{Fake IDirectMusicLoader object. Created the ineffective, easy way...}

interface

uses
  Windows, ActiveX, ComObj, Classes, DirectMusic, Registry;

type

  TinDirectMusicLoader = class(TComObject, IDirectMusicLoader)
  private
    FRealObject: IDirectMusicLoader;
  protected
    function GetObject(const pDesc: TDMus_ObjectDesc;
                        const riid : TGUID;
                        out ppv) : HResult; stdcall;
    function SetObject (const pDesc: TDMus_ObjectDesc) : HResult; stdcall;
    function SetSearchDirectory (const rguidClass: TGUID;
                                 pwzPath: PWideChar;
                                 fClear:  BOOL) : HResult; stdcall;
    function ScanDirectory (const rguidClass: TGUID;
                            pwzFileExtension,
                            pwzScanFileName: PWideChar) : HResult; stdcall;
    function CacheObject (pObject: IDirectMusicObject) : HResult; stdcall;
    function ReleaseObject (pObject: IDirectMusicObject) : HResult; stdcall;
    function ClearCache (const rguidClass: TGUID) : HResult; stdcall;
    function EnableCache (const rguidClass: TGUID;
                          fEnable: BOOL) : HResult; stdcall;
    function EnumObject (const rguidClass: TGUID;
                         dwIndex: DWORD;
                         const pDesc: TDMus_ObjectDesc) : HResult; stdcall;
  public
    procedure Initialize; override;
  end;

  TDllGetClassObject = function (const CLSID, IID: TGUID; var Obj): HResult; stdcall;

const
  Class_IDMLoader: TGUID = '{D2AC2892-B39B-11D1-8704-00600893B1BD}';


implementation

uses ComServ, sysutils, config, packs;


var
  OrigGetClassObject: function (const CLSID, IID: TGUID; var Obj): HResult; stdcall;

  MusicSets: TStringList;
  Soundfont: String;
  MusicPath: String;

  DllHandle: THandle;

{ TTinDirectMusicLoader }

////////////////////////////////////////////////////////////////////////////////

function TinDirectMusicLoader.GetObject(const pDesc: TDMus_ObjectDesc;
  const riid: TGUID; out ppv): HResult;

  procedure FakeLoad(Path: String);
  var
    Desc: TDMus_ObjectDesc;
    FilenameChars : array [0..DMUS_MAX_FILENAME-1] of Char;
  begin
    if Path[2] <> ':' then
      Path:=MusicPath + Path;

    Desc:=pDesc;
    Desc.dwValidData:=Desc.dwValidData or DMUS_OBJ_FULLPATH;
    StrPCopy(FilenameChars, Path);
    MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, @FilenameChars, -1, Desc.wszFilename, MAX_PATH);
    Result:=FRealObject.GetObject(Desc, riid, ppv)
  end;

  procedure RealLoad;
  begin
    Result:=FRealObject.GetObject(pDesc, riid, ppv);
  end;

var
  S: String;
  I: Integer;
begin
  if IsEqualIID(pDesc.guidClass, CLSID_DirectMusicSegment) then
  begin
    S:=pDesc.wszFileName;
    S:=ExtractFileName(S);  //DELETE THIS!!!
    I:=MusicSets.IndexOfName(S);
    if I >= 0 then
      FakeLoad(MusicSets.Values[S])
    else
      RealLoad;
  end
  else if (Soundfont <> '') and IsEqualIID(pDesc.guidClass, CLSID_DirectMusicCollection) then
    FakeLoad(Soundfont)
  else
    RealLoad;
end;

////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////

function TinDirectMusicLoader.CacheObject(pObject: IDirectMusicObject): HResult;
begin
  Result:=FRealObject.CacheObject(pObject);
end;

function TinDirectMusicLoader.ClearCache(const rguidClass: TGUID): HResult;
begin
  Result:=FRealObject.ClearCache(rguidClass);
end;

function TinDirectMusicLoader.EnableCache(const rguidClass: TGUID;
  fEnable: BOOL): HResult;
begin
  Result:=FRealObject.EnableCache(rguidClass, fEnable);
end;

function TinDirectMusicLoader.EnumObject(const rguidClass: TGUID; dwIndex: DWORD;
  const pDesc: TDMus_ObjectDesc): HResult;
begin
  Result:=FRealObject.EnumObject(rguidClass, dwIndex, pDesc);
end;

procedure TinDirectMusicLoader.Initialize;
var
  DLoaderFactory: IClassFactory;
begin
  OrigGetClassObject(CLSID_DirectMusicLoader, IClassFactory, DLoaderFactory);
  DLoaderFactory.CreateInstance(nil, IDirectMusicLoader, FRealObject);
end;

function TinDirectMusicLoader.ReleaseObject(pObject: IDirectMusicObject): HResult;
begin
  Result:=FRealObject.ReleaseObject(pObject);
end;

function TinDirectMusicLoader.ScanDirectory(const rguidClass: TGUID;
  pwzFileExtension, pwzScanFileName: PWideChar): HResult;
begin
  Result:=FRealObject.ScanDirectory(rguidClass, pwzFileExtension, pwzScanFileName);
end;

function TinDirectMusicLoader.SetObject(const pDesc: TDMus_ObjectDesc): HResult;
begin
  Result:=FRealObject.SetObject(pDesc);
end;

function TinDirectMusicLoader.SetSearchDirectory(const rguidClass: TGUID;
  pwzPath: PWideChar; fClear: BOOL): HResult;
begin
  Result:=FRealObject.SetSearchDirectory(rguidClass, pwzPath, fClear);
end;

////////////////////////////////////////////////////////////////////////////////

procedure Init;

  procedure DLLError;
  begin
//    MessageBox(0, 'Can''t find DirectMusicLoader. Check that DirectX is correctly installed.', 'Final Fantasy VIII Configurator', MB_OK or MB_ICONSTOP);
    Halt(GetLastError);
  end;

var
  SGTSets,
  ConfigPath,
  DLLPath: String;
  S, S2: String;
  I, P: Integer;
begin
  SetLength(ConfigPath, MAX_PATH);
  GetModuleFilename(GetModuleHandle('ffaddon'), PChar(ConfigPath), MAX_PATH);
  SetLength(ConfigPath, StrLen(PChar(ConfigPath)));
  ConfigPath:=ExtractFilePath(ConfigPath);

  MusicSets:=TStringList.Create;
  MusicPath:=ExtractFilePath(ParamStr(0));

  with TFF8ConfigIni.Create(ConfigPath + 'config.ini') do
  try
    MusicPath:=AddonPath;
    Soundfont:=StringSettings[fsSoundfont];
    DLLPath:=ReadString(CLSIDSection, GUIDToString(CLSID_DirectMusicLoader), 'dmloader.dll');
    SGTSets:=StringSettings[fsMusicSets];
  finally
    Free;
  end;

  with TPackManager.Create(MusicPath + 'packages.ini') do
  try
    GetSGTOverrides(MusicSets, SGTSets);
    Soundfont:=SFFile[SFPacks.IndexOfName(Soundfont)];
  finally
    Free;
  end;
  if LowerCase(ExtractFileExt(Soundfont)) <> '.dls' then
    Soundfont:='';

  //Remove SGT duplicates
  for I:=0 to MusicSets.Count - 1 do
  begin
    S:=MusicSets.Values[MusicSets.Names[I]];
    repeat
      P:=Pos('|', S);
      if P = 0 then
      begin
        MusicSets.Values[MusicSets.Names[I]]:=S;
        Break;
      end;
      S2:=Copy(S, 1, P - 1);
      if FileExists(MusicPath + S2) then
      begin
        MusicSets.Values[MusicSets.Names[I]]:=S2;
        Break;
      end
      else
        Delete(S, 1, P);
    until S = '';
  end;

  DllHandle:=LoadLibrary(PChar(DllPath));
  if DllHandle = 0 then DllError;
  @OrigGetClassObject:=GetProcAddress(DllHandle, 'DllGetClassObject');
  if @OrigGetClassObject = nil then DllError;
end;

procedure Finit;
begin
  FreeLibrary(DllHandle);
  MusicSets.Free;
end;

initialization
  Init;

  TComObjectFactory.Create(ComServer, TinDirectMusicLoader, Class_IDMLoader,
    'IDMLoader', '', ciMultiInstance, tmApartment);

finalization
  Finit;
end.
